@@ -40,7 +40,7 @@ void Clip::init_settings()
 	gravity = GRAVITY_CENTER;
 	scale = SCALE_FIT;
 	anchor = ANCHOR_CANVAS;
-	handles = TRANSFORM_HANDLE_NONE;
+	display = FRAME_DISPLAY_NONE;
 	waveform = false;
 	previous_properties = "";
 
@@ -57,7 +57,7 @@ void Clip::init_settings()
 	rotation = Keyframe(0.0);
 
 	// Init time & volume
-	time = Keyframe(0.0);
+	time = Keyframe(1.0);
 	volume = Keyframe(1.0);
 
 	// Init audio waveform color
@@ -192,7 +192,7 @@ void Clip::Reader(ReaderBase* new_reader)
 }
 
 /// Get the current reader
-ReaderBase* Clip::Reader() throw(ReaderClosed)
+ReaderBase* Clip::Reader()
 {
 	if (reader)
 		return reader;
@@ -202,7 +202,7 @@ ReaderBase* Clip::Reader() throw(ReaderClosed)
 }
 
 // Open the internal reader
-void Clip::Open() throw(InvalidFile, ReaderClosed)
+void Clip::Open()
 {
 	if (reader)
 	{
@@ -219,7 +219,7 @@ void Clip::Open() throw(InvalidFile, ReaderClosed)
 }
 
 // Close the internal reader
-void Clip::Close() throw(ReaderClosed)
+void Clip::Close()
 {
 	if (reader) {
 		ZmqLogger::Instance()->AppendDebugMethod("Clip::Close", "", -1, "", -1, "", -1, "", -1, "", -1, "", -1);
@@ -233,7 +233,7 @@ void Clip::Close() throw(ReaderClosed)
 }
 
 // Get end position of clip (trim end of video), which can be affected by the time curve.
-float Clip::End() throw(ReaderClosed)
+float Clip::End()
 {
 	// if a time curve is present, use it's length
 	if (time.Points.size() > 1)
@@ -255,7 +255,7 @@ float Clip::End() throw(ReaderClosed)
 }
 
 // Get an openshot::Frame object for a specific frame number of this reader.
-tr1::shared_ptr<Frame> Clip::GetFrame(long int requested_frame) throw(ReaderClosed)
+std::shared_ptr<Frame> Clip::GetFrame(int64_t requested_frame)
 {
 	if (reader)
 	{
@@ -274,30 +274,28 @@ tr1::shared_ptr<Frame> Clip::GetFrame(long int requested_frame) throw(ReaderClos
 		else if (enabled_video == -1 && reader && !reader->info.has_audio)
 			enabled_video = 0;
 
-		// Adjust parent reader with same settings (for performance gains)
-		if (reader) {
-			// Override parent reader
-			reader->info.has_audio = enabled_audio;
-			reader->info.has_video = enabled_video;
-		}
-
 		// Is a time map detected
-		long int new_frame_number = requested_frame;
+		int64_t new_frame_number = requested_frame;
+		int64_t time_mapped_number = adjust_frame_number_minimum(time.GetLong(requested_frame));
 		if (time.Values.size() > 1)
-			new_frame_number = time.GetLong(requested_frame);
-
+            new_frame_number = time_mapped_number;
 
 		// Now that we have re-mapped what frame number is needed, go and get the frame pointer
-		tr1::shared_ptr<Frame> original_frame = GetOrCreateFrame(new_frame_number);
+		std::shared_ptr<Frame> original_frame;
+		#pragma omp critical (Clip_GetFrame)
+		original_frame = GetOrCreateFrame(new_frame_number);
 
 		// Create a new frame
-		tr1::shared_ptr<Frame> frame(new Frame(new_frame_number, 1, 1, "#000000", original_frame->GetAudioSamplesCount(), original_frame->GetAudioChannelsCount()));
-		frame->SampleRate(original_frame->SampleRate());
-		frame->ChannelsLayout(original_frame->ChannelsLayout());
+		std::shared_ptr<Frame> frame(new Frame(new_frame_number, 1, 1, "#000000", original_frame->GetAudioSamplesCount(), original_frame->GetAudioChannelsCount()));
+		#pragma omp critical (Clip_GetFrame)
+		{
+			frame->SampleRate(original_frame->SampleRate());
+			frame->ChannelsLayout(original_frame->ChannelsLayout());
+		}
 
 		// Copy the image from the odd field
 		if (enabled_video)
-			frame->AddImage(tr1::shared_ptr<QImage>(new QImage(*original_frame->GetImage())));
+			frame->AddImage(std::shared_ptr<QImage>(new QImage(*original_frame->GetImage())));
 
 		// Loop through each channel, add audio
 		if (enabled_audio && reader->info.has_audio)
@@ -305,7 +303,7 @@ tr1::shared_ptr<Frame> Clip::GetFrame(long int requested_frame) throw(ReaderClos
 				frame->AddAudio(true, channel, 0, original_frame->GetAudioSamples(channel), original_frame->GetAudioSamplesCount(), 1.0);
 
 		// Get time mapped frame number (used to increase speed, change direction, etc...)
-		tr1::shared_ptr<Frame> new_frame = get_time_mapped_frame(frame, requested_frame);
+		std::shared_ptr<Frame> new_frame = get_time_mapped_frame(frame, requested_frame);
 
 		// Apply effects to the frame (if any)
 		apply_effects(new_frame);
@@ -354,7 +352,7 @@ void Clip::reverse_buffer(juce::AudioSampleBuffer* buffer)
 }
 
 // Adjust the audio and image of a time mapped frame
-tr1::shared_ptr<Frame> Clip::get_time_mapped_frame(tr1::shared_ptr<Frame> frame, long int frame_number) throw(ReaderClosed)
+std::shared_ptr<Frame> Clip::get_time_mapped_frame(std::shared_ptr<Frame> frame, int64_t frame_number)
 {
 	// Check for valid reader
 	if (!reader)
@@ -364,7 +362,8 @@ tr1::shared_ptr<Frame> Clip::get_time_mapped_frame(tr1::shared_ptr<Frame> frame,
 	// Check for a valid time map curve
 	if (time.Values.size() > 1)
 	{
-		tr1::shared_ptr<Frame> new_frame;
+		const GenericScopedLock<CriticalSection> lock(getFrameCriticalSection);
+		std::shared_ptr<Frame> new_frame;
 
 		// create buffer and resampler
 		juce::AudioSampleBuffer *samples = NULL;
@@ -372,11 +371,11 @@ tr1::shared_ptr<Frame> Clip::get_time_mapped_frame(tr1::shared_ptr<Frame> frame,
 			resampler = new AudioResampler();
 
 		// Get new frame number
-		int new_frame_number = round(time.GetValue(frame_number));
+		int new_frame_number = adjust_frame_number_minimum(round(time.GetValue(frame_number)));
 
 		// Create a new frame
 		int samples_in_frame = Frame::GetSamplesPerFrame(new_frame_number, reader->info.fps, reader->info.sample_rate, frame->GetAudioChannelsCount());
-		new_frame = tr1::shared_ptr<Frame>(new Frame(new_frame_number, 1, 1, "#000000", samples_in_frame, frame->GetAudioChannelsCount()));
+		new_frame = std::make_shared<Frame>(new_frame_number, 1, 1, "#000000", samples_in_frame, frame->GetAudioChannelsCount());
 
 		// Copy the image from the new frame
 		new_frame->AddImage(GetOrCreateFrame(new_frame_number)->GetImage());
@@ -575,7 +574,7 @@ tr1::shared_ptr<Frame> Clip::get_time_mapped_frame(tr1::shared_ptr<Frame> frame,
 }
 
 // Adjust frame number minimum value
-long int Clip::adjust_frame_number_minimum(long int frame_number)
+int64_t Clip::adjust_frame_number_minimum(int64_t frame_number)
 {
 	// Never return a frame number 0 or below
 	if (frame_number < 1)
@@ -586,9 +585,9 @@ long int Clip::adjust_frame_number_minimum(long int frame_number)
 }
 
 // Get or generate a blank frame
-tr1::shared_ptr<Frame> Clip::GetOrCreateFrame(long int number)
+std::shared_ptr<Frame> Clip::GetOrCreateFrame(int64_t number)
 {
-	tr1::shared_ptr<Frame> new_frame;
+	std::shared_ptr<Frame> new_frame;
 
 	// Init some basic properties about this frame
 	int samples_in_frame = Frame::GetSamplesPerFrame(number, reader->info.fps, reader->info.sample_rate, reader->info.channels);
@@ -599,12 +598,14 @@ tr1::shared_ptr<Frame> Clip::GetOrCreateFrame(long int number)
 
 		// Determine the max size of this clips source image (based on the timeline's size, the scaling mode,
 		// and the scaling keyframes). This is a performance improvement, to keep the images as small as possible,
-		// without loosing quality.
+		// without loosing quality. NOTE: We cannot go smaller than the timeline itself, or the add_layer timeline
+        // method will scale it back to timeline size before scaling it smaller again. This needs to be fixed in
+        // the future.
 		if (scale == SCALE_FIT || scale == SCALE_STRETCH) {
 			// Best fit or Stretch scaling (based on max timeline size * scaling keyframes)
 			float max_scale_x = scale_x.GetMaxPoint().co.Y;
 			float max_scale_y = scale_y.GetMaxPoint().co.Y;
-			reader->SetMaxSize(max_width * max_scale_x, max_height * max_scale_y);
+			reader->SetMaxSize(max(float(max_width), max_width * max_scale_x), max(float(max_height), max_height * max_scale_y));
 
 		} else if (scale == SCALE_CROP) {
 			// Cropping scale mode (based on max timeline size * cropped size * scaling keyframes)
@@ -615,9 +616,9 @@ tr1::shared_ptr<Frame> Clip::GetOrCreateFrame(long int number)
 
 			// respect aspect ratio
 			if (width_size.width() >= max_width && width_size.height() >= max_height)
-				reader->SetMaxSize(width_size.width(), width_size.height());
+				reader->SetMaxSize(max(max_width, width_size.width()), max(max_height, width_size.height()));
 			else
-				reader->SetMaxSize(height_size.width(), height_size.height());
+				reader->SetMaxSize(max(max_width, height_size.width()), max(max_height, height_size.height()));
 
 		} else {
 			// No scaling, use original image size (slower)
@@ -643,7 +644,7 @@ tr1::shared_ptr<Frame> Clip::GetOrCreateFrame(long int number)
 	ZmqLogger::Instance()->AppendDebugMethod("Clip::GetOrCreateFrame (create blank)", "number", number, "samples_in_frame", samples_in_frame, "", -1, "", -1, "", -1, "", -1);
 
 	// Create blank frame
-	new_frame = tr1::shared_ptr<Frame>(new Frame(number, reader->info.width, reader->info.height, "#000000", samples_in_frame, reader->info.channels));
+	new_frame = std::make_shared<Frame>(number, reader->info.width, reader->info.height, "#000000", samples_in_frame, reader->info.channels);
 	new_frame->SampleRate(reader->info.sample_rate);
 	new_frame->ChannelsLayout(reader->info.channel_layout);
 	return new_frame;
@@ -657,7 +658,7 @@ string Clip::Json() {
 }
 
 // Get all properties for a specific frame
-string Clip::PropertiesJSON(long int requested_frame) {
+string Clip::PropertiesJSON(int64_t requested_frame) {
 
 	// Generate JSON properties list
 	Json::Value root;
@@ -670,7 +671,7 @@ string Clip::PropertiesJSON(long int requested_frame) {
 	root["gravity"] = add_property_json("Gravity", gravity, "int", "", NULL, 0, 8, false, requested_frame);
 	root["scale"] = add_property_json("Scale", scale, "int", "", NULL, 0, 3, false, requested_frame);
 	root["anchor"] = add_property_json("Anchor", anchor, "int", "", NULL, 0, 1, false, requested_frame);
-	root["handles"] = add_property_json("Handles", handles, "int", "", NULL, 0, 1, false, requested_frame);
+	root["display"] = add_property_json("Frame Number", display, "int", "", NULL, 0, 3, false, requested_frame);
 	root["waveform"] = add_property_json("Waveform", waveform, "int", "", NULL, 0, 1, false, requested_frame);
 
 	// Add gravity choices (dropdown style)
@@ -694,9 +695,11 @@ string Clip::PropertiesJSON(long int requested_frame) {
 	root["anchor"]["choices"].append(add_property_choice_json("Canvas", ANCHOR_CANVAS, anchor));
 	root["anchor"]["choices"].append(add_property_choice_json("Viewport", ANCHOR_VIEWPORT, anchor));
 
-	// Add transform handles choices (dropdown style)
-	root["handles"]["choices"].append(add_property_choice_json("None", TRANSFORM_HANDLE_NONE, handles));
-	root["handles"]["choices"].append(add_property_choice_json("Selection", TRANSFORM_HANDLE_SELECTION, handles));
+	// Add frame number display choices (dropdown style)
+	root["display"]["choices"].append(add_property_choice_json("None", FRAME_DISPLAY_NONE, display));
+	root["display"]["choices"].append(add_property_choice_json("Clip", FRAME_DISPLAY_CLIP, display));
+	root["display"]["choices"].append(add_property_choice_json("Timeline", FRAME_DISPLAY_TIMELINE, display));
+	root["display"]["choices"].append(add_property_choice_json("Both", FRAME_DISPLAY_BOTH, display));
 
 	// Add waveform choices (dropdown style)
 	root["waveform"]["choices"].append(add_property_choice_json("Yes", true, waveform));
@@ -736,7 +739,7 @@ Json::Value Clip::JsonValue() {
 	root["gravity"] = gravity;
 	root["scale"] = scale;
 	root["anchor"] = anchor;
-	root["handles"] = handles;
+	root["display"] = display;
 	root["waveform"] = waveform;
 	root["scale_x"] = scale_x.JsonValue();
 	root["scale_y"] = scale_y.JsonValue();
@@ -786,7 +789,7 @@ Json::Value Clip::JsonValue() {
 }
 
 // Load JSON string into this object
-void Clip::SetJson(string value) throw(InvalidJSON) {
+void Clip::SetJson(string value) {
 
 	// Parse JSON string into JSON objects
 	Json::Value root;
@@ -821,8 +824,8 @@ void Clip::SetJsonValue(Json::Value root) {
 		scale = (ScaleType) root["scale"].asInt();
 	if (!root["anchor"].isNull())
 		anchor = (AnchorType) root["anchor"].asInt();
-	if (!root["handles"].isNull())
-		handles = (TransformHandleType) root["handles"].asInt();
+	if (!root["display"].isNull())
+		display = (FrameDisplayType) root["display"].asInt();
 	if (!root["waveform"].isNull())
 		waveform = root["waveform"].asBool();
 	if (!root["scale_x"].isNull())
@@ -999,7 +1002,7 @@ void Clip::RemoveEffect(EffectBase* effect)
 }
 
 // Apply effects to the source frame (if any)
-tr1::shared_ptr<Frame> Clip::apply_effects(tr1::shared_ptr<Frame> frame)
+std::shared_ptr<Frame> Clip::apply_effects(std::shared_ptr<Frame> frame)
 {
 	// Find Effects at this position and layer
 	list<EffectBase*>::iterator effect_itr;
