@@ -35,43 +35,43 @@ Frame::Frame() : number(1), pixel_ratio(1,1), channels(2), width(1), height(1),
 		channel_layout(LAYOUT_STEREO), sample_rate(44100), qbuffer(NULL), has_audio_data(false), has_image_data(false)
 {
 	// Init the image magic and audio buffer
-	audio = tr1::shared_ptr<juce::AudioSampleBuffer>(new juce::AudioSampleBuffer(channels, 0));
+	audio = std::shared_ptr<juce::AudioSampleBuffer>(new juce::AudioSampleBuffer(channels, 0));
 
 	// initialize the audio samples to zero (silence)
 	audio->clear();
 };
 
 // Constructor - image only (48kHz audio silence)
-Frame::Frame(long int number, int width, int height, string color)
+Frame::Frame(int64_t number, int width, int height, string color)
 	: number(number), pixel_ratio(1,1), channels(2), width(width), height(height),
 	  channel_layout(LAYOUT_STEREO), sample_rate(44100), qbuffer(NULL), has_audio_data(false), has_image_data(false)
 {
 	// Init the image magic and audio buffer
-	audio = tr1::shared_ptr<juce::AudioSampleBuffer>(new juce::AudioSampleBuffer(channels, 0));
+	audio = std::shared_ptr<juce::AudioSampleBuffer>(new juce::AudioSampleBuffer(channels, 0));
 
 	// initialize the audio samples to zero (silence)
 	audio->clear();
 };
 
 // Constructor - audio only (300x200 blank image)
-Frame::Frame(long int number, int samples, int channels) :
+Frame::Frame(int64_t number, int samples, int channels) :
 		number(number), pixel_ratio(1,1), channels(channels), width(1), height(1),
 		channel_layout(LAYOUT_STEREO), sample_rate(44100), qbuffer(NULL), has_audio_data(false), has_image_data(false)
 {
 	// Init the image magic and audio buffer
-	audio = tr1::shared_ptr<juce::AudioSampleBuffer>(new juce::AudioSampleBuffer(channels, samples));
+	audio = std::shared_ptr<juce::AudioSampleBuffer>(new juce::AudioSampleBuffer(channels, samples));
 
 	// initialize the audio samples to zero (silence)
 	audio->clear();
 };
 
 // Constructor - image & audio
-Frame::Frame(long int number, int width, int height, string color, int samples, int channels)
+Frame::Frame(int64_t number, int width, int height, string color, int samples, int channels)
 	: number(number), pixel_ratio(1,1), channels(channels), width(width), height(height),
 	  channel_layout(LAYOUT_STEREO), sample_rate(44100), qbuffer(NULL), has_audio_data(false), has_image_data(false)
 {
 	// Init the image magic and audio buffer
-	audio = tr1::shared_ptr<juce::AudioSampleBuffer>(new juce::AudioSampleBuffer(channels, samples));
+	audio = std::shared_ptr<juce::AudioSampleBuffer>(new juce::AudioSampleBuffer(channels, samples));
 
 	// initialize the audio samples to zero (silence)
 	audio->clear();
@@ -89,8 +89,8 @@ Frame::Frame ( const Frame &other )
 void Frame::DeepCopy(const Frame& other)
 {
 	number = other.number;
-	image = tr1::shared_ptr<QImage>(new QImage(*(other.image)));
-	audio = tr1::shared_ptr<juce::AudioSampleBuffer>(new juce::AudioSampleBuffer(*(other.audio)));
+	image = std::shared_ptr<QImage>(new QImage(*(other.image)));
+	audio = std::shared_ptr<juce::AudioSampleBuffer>(new juce::AudioSampleBuffer(*(other.audio)));
 	pixel_ratio = Fraction(other.pixel_ratio.num, other.pixel_ratio.den);
 	channels = other.channels;
 	channel_layout = other.channel_layout;
@@ -100,7 +100,7 @@ void Frame::DeepCopy(const Frame& other)
 
 
 	if (other.wave_image)
-		wave_image = tr1::shared_ptr<QImage>(new QImage(*(other.wave_image)));
+		wave_image = std::shared_ptr<QImage>(new QImage(*(other.wave_image)));
 }
 
 // Descructor
@@ -117,11 +117,11 @@ void Frame::Display()
 		// Only create the QApplication once
 		static int argc = 1;
 		static char* argv[1] = {NULL};
-		previewApp = tr1::shared_ptr<QApplication>(new QApplication(argc, argv));
+		previewApp = std::shared_ptr<QApplication>(new QApplication(argc, argv));
 	}
 
 	// Get preview image
-	tr1::shared_ptr<QImage> previewImage = GetImage();
+	std::shared_ptr<QImage> previewImage = GetImage();
 
 	// Update the image to reflect the correct pixel aspect ration (i.e. to fix non-squar pixels)
 	if (pixel_ratio.num != 1 || pixel_ratio.den != 1)
@@ -131,7 +131,7 @@ void Frame::Display()
 		int new_height = previewImage->size().height() * pixel_ratio.Reciprocal().ToDouble();
 
 		// Resize to fix DAR
-		previewImage = tr1::shared_ptr<QImage>(new QImage(previewImage->scaled(new_width, new_height, Qt::IgnoreAspectRatio, Qt::SmoothTransformation)));
+		previewImage = std::shared_ptr<QImage>(new QImage(previewImage->scaled(new_width, new_height, Qt::IgnoreAspectRatio, Qt::SmoothTransformation)));
 	}
 
 	// Create window
@@ -152,7 +152,7 @@ void Frame::Display()
 }
 
 // Get an audio waveform image
-tr1::shared_ptr<QImage> Frame::GetWaveform(int width, int height, int Red, int Green, int Blue, int Alpha)
+std::shared_ptr<QImage> Frame::GetWaveform(int width, int height, int Red, int Green, int Blue, int Alpha)
 {
 	// Clear any existing waveform image
 	ClearWaveform();
@@ -207,7 +207,7 @@ tr1::shared_ptr<QImage> Frame::GetWaveform(int width, int height, int Red, int G
 		}
 
 		// Create blank image
-		wave_image = tr1::shared_ptr<QImage>(new QImage(total_width, total_height, QImage::Format_RGBA8888));
+		wave_image = std::shared_ptr<QImage>(new QImage(total_width, total_height, QImage::Format_RGBA8888));
 		wave_image->fill(QColor(0,0,0,0));
 
 		// Load QPainter with wave_image device
@@ -232,13 +232,13 @@ tr1::shared_ptr<QImage> Frame::GetWaveform(int width, int height, int Red, int G
 		// Resize Image (if requested)
 		if (width != total_width || height != total_height) {
 			QImage scaled_wave_image = wave_image->scaled(width, height, Qt::IgnoreAspectRatio, Qt::FastTransformation);
-			wave_image = tr1::shared_ptr<QImage>(new QImage(scaled_wave_image));
+			wave_image = std::shared_ptr<QImage>(new QImage(scaled_wave_image));
 		}
 	}
 	else
 	{
 		// No audio samples present
-		wave_image = tr1::shared_ptr<QImage>(new QImage(width, height, QImage::Format_RGBA8888));
+		wave_image = std::shared_ptr<QImage>(new QImage(width, height, QImage::Format_RGBA8888));
 		wave_image->fill(QColor(QString::fromStdString("#000000")));
 	}
 
@@ -273,7 +273,7 @@ void Frame::DisplayWaveform()
 		// Only create the QApplication once
 		static int argc = 1;
 		static char* argv[1] = {NULL};
-		previewApp = tr1::shared_ptr<QApplication>(new QApplication(argc, argv));
+		previewApp = std::shared_ptr<QApplication>(new QApplication(argc, argv));
 	}
 
 	// Create window
@@ -410,13 +410,21 @@ float* Frame::GetInterleavedAudioSamples(int new_sample_rate, AudioResampler* re
 // Get number of audio channels
 int Frame::GetAudioChannelsCount()
 {
-	return audio->getNumChannels();
+    const GenericScopedLock<CriticalSection> lock(addingAudioSection);
+	if (audio)
+		return audio->getNumChannels();
+	else
+		return 0;
 }
 
 // Get number of audio samples
 int Frame::GetAudioSamplesCount()
 {
-	return audio->getNumSamples();
+    const GenericScopedLock<CriticalSection> lock(addingAudioSection);
+	if (audio)
+		return audio->getNumSamples();
+	else
+		return 0;
 }
 
 juce::AudioSampleBuffer *Frame::GetAudioSampleBuffer()
@@ -425,9 +433,9 @@ juce::AudioSampleBuffer *Frame::GetAudioSampleBuffer()
 }
 
 // Get the size in bytes of this frame (rough estimate)
-int64 Frame::GetBytes()
+int64_t Frame::GetBytes()
 {
-	int64 total_bytes = 0;
+	int64_t total_bytes = 0;
 	if (image)
 		total_bytes += (width * height * sizeof(char) * 4);
 	if (audio) {
@@ -466,13 +474,13 @@ void Frame::SetPixelRatio(int num, int den)
 }
 
 // Set frame number
-void Frame::SetFrameNumber(int new_number)
+void Frame::SetFrameNumber(int64_t new_number)
 {
 	number = new_number;
 }
 
 // Calculate the # of samples per video frame (for a specific frame number and frame rate)
-int Frame::GetSamplesPerFrame(long int number, Fraction fps, int sample_rate, int channels)
+int Frame::GetSamplesPerFrame(int64_t number, Fraction fps, int sample_rate, int channels)
 {
 	// Get the total # of samples for the previous frame, and the current frame (rounded)
 	double fps_rate = fps.Reciprocal().ToDouble();
@@ -528,7 +536,7 @@ ChannelLayout Frame::ChannelsLayout()
 void Frame::Save(string path, float scale, string format, int quality)
 {
 	// Get preview image
-	tr1::shared_ptr<QImage> previewImage = GetImage();
+	std::shared_ptr<QImage> previewImage = GetImage();
 
 	// scale image if needed
 	if (abs(scale) > 1.001 || abs(scale) < 0.999)
@@ -544,11 +552,11 @@ void Frame::Save(string path, float scale, string format, int quality)
 			int new_height = previewImage->size().height() * pixel_ratio.Reciprocal().ToDouble();
 
 			// Resize to fix DAR
-			previewImage = tr1::shared_ptr<QImage>(new QImage(previewImage->scaled(new_width, new_height, Qt::IgnoreAspectRatio, Qt::SmoothTransformation)));
+			previewImage = std::shared_ptr<QImage>(new QImage(previewImage->scaled(new_width, new_height, Qt::IgnoreAspectRatio, Qt::SmoothTransformation)));
 		}
 
 		// Resize image
-		previewImage = tr1::shared_ptr<QImage>(new QImage(previewImage->scaled(new_width * scale, new_height * scale, Qt::KeepAspectRatio, Qt::SmoothTransformation)));
+		previewImage = std::shared_ptr<QImage>(new QImage(previewImage->scaled(new_width * scale, new_height * scale, Qt::KeepAspectRatio, Qt::SmoothTransformation)));
 	}
 
 	// Save image
@@ -557,10 +565,10 @@ void Frame::Save(string path, float scale, string format, int quality)
 
 // Thumbnail the frame image to the specified path.  The image format is determined from the extension (i.e. image.PNG, image.JPEG)
 void Frame::Thumbnail(string path, int new_width, int new_height, string mask_path, string overlay_path,
-		string background_color, bool ignore_aspect, string format, int quality) throw(InvalidFile) {
+		string background_color, bool ignore_aspect, string format, int quality) {
 
 	// Create blank thumbnail image & fill background color
-	tr1::shared_ptr<QImage> thumbnail = tr1::shared_ptr<QImage>(new QImage(new_width, new_height, QImage::Format_RGBA8888));
+	std::shared_ptr<QImage> thumbnail = std::shared_ptr<QImage>(new QImage(new_width, new_height, QImage::Format_RGBA8888));
 	thumbnail->fill(QColor(QString::fromStdString(background_color)));
 
 	// Create transform and painter
@@ -570,7 +578,7 @@ void Frame::Thumbnail(string path, int new_width, int new_height, string mask_pa
 
 
 	// Get preview image
-	tr1::shared_ptr<QImage> previewImage = GetImage();
+	std::shared_ptr<QImage> previewImage = GetImage();
 
 	// Update the image to reflect the correct pixel aspect ration (i.e. to fix non-squar pixels)
 	if (pixel_ratio.num != 1 || pixel_ratio.den != 1)
@@ -580,16 +588,16 @@ void Frame::Thumbnail(string path, int new_width, int new_height, string mask_pa
 		int aspect_height = previewImage->size().height() * pixel_ratio.Reciprocal().ToDouble();
 
 		// Resize to fix DAR
-		previewImage = tr1::shared_ptr<QImage>(new QImage(previewImage->scaled(aspect_width, aspect_height, Qt::IgnoreAspectRatio, Qt::SmoothTransformation)));
+		previewImage = std::shared_ptr<QImage>(new QImage(previewImage->scaled(aspect_width, aspect_height, Qt::IgnoreAspectRatio, Qt::SmoothTransformation)));
 	}
 
 	// Resize frame image
 	if (ignore_aspect)
 		// Ignore aspect ratio
-		previewImage = tr1::shared_ptr<QImage>(new QImage(previewImage->scaled(new_width, new_height, Qt::IgnoreAspectRatio, Qt::SmoothTransformation)));
+		previewImage = std::shared_ptr<QImage>(new QImage(previewImage->scaled(new_width, new_height, Qt::IgnoreAspectRatio, Qt::SmoothTransformation)));
 	else
 		// Maintain aspect ratio
-		previewImage = tr1::shared_ptr<QImage>(new QImage(previewImage->scaled(new_width, new_height, Qt::KeepAspectRatio, Qt::SmoothTransformation)));
+		previewImage = std::shared_ptr<QImage>(new QImage(previewImage->scaled(new_width, new_height, Qt::KeepAspectRatio, Qt::SmoothTransformation)));
 
 	// Composite frame image onto background (centered)
 	int x = (new_width - previewImage->size().width()) / 2.0; // center
@@ -601,14 +609,14 @@ void Frame::Thumbnail(string path, int new_width, int new_height, string mask_pa
 	// Overlay Image (if any)
 	if (overlay_path != "") {
 		// Open overlay
-		tr1::shared_ptr<QImage> overlay = tr1::shared_ptr<QImage>(new QImage());
+		std::shared_ptr<QImage> overlay = std::shared_ptr<QImage>(new QImage());
 		overlay->load(QString::fromStdString(overlay_path));
 
 		// Set pixel format
-		overlay = tr1::shared_ptr<QImage>(new QImage(overlay->convertToFormat(QImage::Format_RGBA8888)));
+		overlay = std::shared_ptr<QImage>(new QImage(overlay->convertToFormat(QImage::Format_RGBA8888)));
 
 		// Resize to fit
-		overlay = tr1::shared_ptr<QImage>(new QImage(overlay->scaled(new_width, new_height, Qt::IgnoreAspectRatio, Qt::SmoothTransformation)));
+		overlay = std::shared_ptr<QImage>(new QImage(overlay->scaled(new_width, new_height, Qt::IgnoreAspectRatio, Qt::SmoothTransformation)));
 
 		// Composite onto thumbnail
 		painter.setCompositionMode(QPainter::CompositionMode_SourceOver);
@@ -619,14 +627,14 @@ void Frame::Thumbnail(string path, int new_width, int new_height, string mask_pa
 	// Mask Image (if any)
 	if (mask_path != "") {
 		// Open mask
-		tr1::shared_ptr<QImage> mask = tr1::shared_ptr<QImage>(new QImage());
+		std::shared_ptr<QImage> mask = std::shared_ptr<QImage>(new QImage());
 		mask->load(QString::fromStdString(mask_path));
 
 		// Set pixel format
-		mask = tr1::shared_ptr<QImage>(new QImage(mask->convertToFormat(QImage::Format_RGBA8888)));
+		mask = std::shared_ptr<QImage>(new QImage(mask->convertToFormat(QImage::Format_RGBA8888)));
 
 		// Resize to fit
-		mask = tr1::shared_ptr<QImage>(new QImage(mask->scaled(new_width, new_height, Qt::IgnoreAspectRatio, Qt::SmoothTransformation)));
+		mask = std::shared_ptr<QImage>(new QImage(mask->scaled(new_width, new_height, Qt::IgnoreAspectRatio, Qt::SmoothTransformation)));
 
 		// Negate mask
 		mask->invertPixels();
@@ -674,11 +682,13 @@ void Frame::AddColor(int new_width, int new_height, string color)
 {
 	// Create new image object, and fill with pixel data
 	const GenericScopedLock<CriticalSection> lock(addingImageSection);
-	image = tr1::shared_ptr<QImage>(new QImage(new_width, new_height, QImage::Format_RGBA8888));
-
-	// Fill with solid color
-	image->fill(QColor(QString::fromStdString(color)));
+	#pragma omp critical (AddImage)
+	{
+		image = std::shared_ptr<QImage>(new QImage(new_width, new_height, QImage::Format_RGBA8888));
 
+		// Fill with solid color
+		image->fill(QColor(QString::fromStdString(color)));
+	}
 	// Update height and width
 	width = image->width();
 	height = image->height();
@@ -697,20 +707,23 @@ void Frame::AddImage(int new_width, int new_height, int bytes_per_pixel, QImage:
 	memcpy((unsigned char*)qbuffer, pixels_, buffer_size);
 
 	// Create new image object, and fill with pixel data
-	image = tr1::shared_ptr<QImage>(new QImage(qbuffer, new_width, new_height, new_width * bytes_per_pixel, type, (QImageCleanupFunction) &openshot::Frame::cleanUpBuffer, (void*) qbuffer));
+	#pragma omp critical (AddImage)
+	{
+		image = std::shared_ptr<QImage>(new QImage(qbuffer, new_width, new_height, new_width * bytes_per_pixel, type, (QImageCleanupFunction) &openshot::Frame::cleanUpBuffer, (void*) qbuffer));
 
-	// Always convert to RGBA8888 (if different)
-	if (image->format() != QImage::Format_RGBA8888)
-		image->convertToFormat(QImage::Format_RGBA8888);
+		// Always convert to RGBA8888 (if different)
+		if (image->format() != QImage::Format_RGBA8888)
+			image->convertToFormat(QImage::Format_RGBA8888);
 
-	// Update height and width
-	width = image->width();
-	height = image->height();
-	has_image_data = true;
+			// Update height and width
+		width = image->width();
+		height = image->height();
+		has_image_data = true;
+	}
 }
 
 // Add (or replace) pixel data to the frame
-void Frame::AddImage(tr1::shared_ptr<QImage> new_image)
+void Frame::AddImage(std::shared_ptr<QImage> new_image)
 {
 	// Ignore blank images
 	if (!new_image)
@@ -718,20 +731,23 @@ void Frame::AddImage(tr1::shared_ptr<QImage> new_image)
 
 	// assign image data
 	const GenericScopedLock<CriticalSection> lock(addingImageSection);
-	image = new_image;
+	#pragma omp critical (AddImage)
+	{
+		image = new_image;
 
-	// Always convert to RGBA8888 (if different)
-	if (image->format() != QImage::Format_RGBA8888)
-		image->convertToFormat(QImage::Format_RGBA8888);
+		// Always convert to RGBA8888 (if different)
+		if (image->format() != QImage::Format_RGBA8888)
+			image->convertToFormat(QImage::Format_RGBA8888);
 
-	// Update height and width
-	width = image->width();
-	height = image->height();
-	has_image_data = true;
+		// Update height and width
+		width = image->width();
+		height = image->height();
+		has_image_data = true;
+	}
 }
 
 // Add (or replace) pixel data to the frame (for only the odd or even lines)
-void Frame::AddImage(tr1::shared_ptr<QImage> new_image, bool only_odd_lines)
+void Frame::AddImage(std::shared_ptr<QImage> new_image, bool only_odd_lines)
 {
 	// Ignore blank new_image
 	if (!new_image)
@@ -745,27 +761,34 @@ void Frame::AddImage(tr1::shared_ptr<QImage> new_image, bool only_odd_lines)
 	} else {
 
 		// Ignore image of different sizes or formats
+		bool ret=false;
+		#pragma omp critical (AddImage)
 		if (image == new_image || image->size() != image->size() || image->format() != image->format())
+			ret=true;
+		if (ret)
 			return;
 
 		// Get the frame's image
 		const GenericScopedLock<CriticalSection> lock(addingImageSection);
-		const unsigned char *pixels = image->bits();
-		const unsigned char *new_pixels = new_image->bits();
+		#pragma omp critical (AddImage)
+		{
+			const unsigned char *pixels = image->bits();
+			const unsigned char *new_pixels = new_image->bits();
 
-		// Loop through the scanlines of the image (even or odd)
-		int start = 0;
-		if (only_odd_lines)
-			start = 1;
-		for (int row = start; row < image->height(); row += 2) {
-			memcpy((unsigned char *) pixels, new_pixels + (row * image->bytesPerLine()), image->bytesPerLine());
-			new_pixels += image->bytesPerLine();
+			// Loop through the scanlines of the image (even or odd)
+			int start = 0;
+			if (only_odd_lines)
+				start = 1;
+				for (int row = start; row < image->height(); row += 2) {
+					memcpy((unsigned char *) pixels, new_pixels + (row * image->bytesPerLine()), image->bytesPerLine());
+					new_pixels += image->bytesPerLine();
+				}
+
+			// Update height and width
+			width = image->width();
+			height = image->height();
+			has_image_data = true;
 		}
-
-		// Update height and width
-		width = image->width();
-		height = image->height();
-		has_image_data = true;
 	}
 }
 
@@ -773,41 +796,48 @@ void Frame::AddImage(tr1::shared_ptr<QImage> new_image, bool only_odd_lines)
 // Resize audio container to hold more (or less) samples and channels
 void Frame::ResizeAudio(int channels, int length, int rate, ChannelLayout layout)
 {
-	// Resize JUCE audio buffer
+    const GenericScopedLock<CriticalSection> lock(addingAudioSection);
+
+    // Resize JUCE audio buffer
 	audio->setSize(channels, length, true, true, false);
 	channel_layout = layout;
 	sample_rate = rate;
 }
 
 // Add audio samples to a specific channel
-void Frame::AddAudio(bool replaceSamples, int destChannel, int destStartSample, const float* source, int numSamples, float gainToApplyToSource = 1.0f)
-{
-	// Extend audio container to hold more (or less) samples and channels.. if needed
-	int new_length = destStartSample + numSamples;
-	int new_channel_length = audio->getNumChannels();
-	if (destChannel >= new_channel_length)
-		new_channel_length = destChannel + 1;
-	if (new_length > audio->getNumSamples() || new_channel_length > audio->getNumChannels())
-		audio->setSize(new_channel_length, new_length, true, true, false);
+void Frame::AddAudio(bool replaceSamples, int destChannel, int destStartSample, const float* source, int numSamples, float gainToApplyToSource = 1.0f) {
+	const GenericScopedLock<CriticalSection> lock(addingAudioSection);
+	#pragma omp critical (adding_audio)
+    {
+		// Extend audio container to hold more (or less) samples and channels.. if needed
+		int new_length = destStartSample + numSamples;
+		int new_channel_length = audio->getNumChannels();
+		if (destChannel >= new_channel_length)
+			new_channel_length = destChannel + 1;
+		if (new_length > audio->getNumSamples() || new_channel_length > audio->getNumChannels())
+			audio->setSize(new_channel_length, new_length, true, true, false);
 
-	// Clear the range of samples first (if needed)
-	if (replaceSamples)
-		audio->clear(destChannel, destStartSample, numSamples);
+		// Clear the range of samples first (if needed)
+		if (replaceSamples)
+			audio->clear(destChannel, destStartSample, numSamples);
 
-	// Add samples to frame's audio buffer
-	audio->addFrom(destChannel, destStartSample, source, numSamples, gainToApplyToSource);
-	has_audio_data = true;
+		// Add samples to frame's audio buffer
+		audio->addFrom(destChannel, destStartSample, source, numSamples, gainToApplyToSource);
+		has_audio_data = true;
+	}
 }
 
 // Apply gain ramp (i.e. fading volume)
 void Frame::ApplyGainRamp(int destChannel, int destStartSample, int numSamples, float initial_gain = 0.0f, float final_gain = 1.0f)
 {
-	// Apply gain ramp
+    const GenericScopedLock<CriticalSection> lock(addingAudioSection);
+
+    // Apply gain ramp
 	audio->applyGainRamp(destChannel, destStartSample, numSamples, initial_gain, final_gain);
 }
 
 // Get pointer to Magick++ image object
-tr1::shared_ptr<QImage> Frame::GetImage()
+std::shared_ptr<QImage> Frame::GetImage()
 {
 	// Check for blank image
 	if (!image)
@@ -819,7 +849,7 @@ tr1::shared_ptr<QImage> Frame::GetImage()
 
 #ifdef USE_IMAGEMAGICK
 // Get pointer to ImageMagick image object
-tr1::shared_ptr<Magick::Image> Frame::GetMagickImage()
+std::shared_ptr<Magick::Image> Frame::GetMagickImage()
 {
 	// Check for blank image
 	if (!image)
@@ -830,7 +860,7 @@ tr1::shared_ptr<Magick::Image> Frame::GetMagickImage()
 	QRgb const *tmpBits = (const QRgb*)image->bits();
 
 	// Create new image object, and fill with pixel data
-	tr1::shared_ptr<Magick::Image> magick_image = tr1::shared_ptr<Magick::Image>(new Magick::Image(image->width(), image->height(),"RGBA", Magick::CharPixel, tmpBits));
+	std::shared_ptr<Magick::Image> magick_image = std::shared_ptr<Magick::Image>(new Magick::Image(image->width(), image->height(),"RGBA", Magick::CharPixel, tmpBits));
 
 	// Give image a transparent background color
 	magick_image->backgroundColor(Magick::Color("none"));
@@ -843,7 +873,7 @@ tr1::shared_ptr<Magick::Image> Frame::GetMagickImage()
 
 #ifdef USE_IMAGEMAGICK
 // Get pointer to QImage of frame
-void Frame::AddMagickImage(tr1::shared_ptr<Magick::Image> new_image)
+void Frame::AddMagickImage(std::shared_ptr<Magick::Image> new_image)
 {
 	const int BPP = 4;
 	const std::size_t bufferSize = new_image->columns() * new_image->rows() * BPP;
@@ -867,7 +897,7 @@ void Frame::AddMagickImage(tr1::shared_ptr<Magick::Image> new_image)
     }
 
     // Create QImage of frame data
-    image = tr1::shared_ptr<QImage>(new QImage(qbuffer, width, height, width * BPP, QImage::Format_RGBA8888, (QImageCleanupFunction) &cleanUpBuffer, (void*) qbuffer));
+    image = std::shared_ptr<QImage>(new QImage(qbuffer, width, height, width * BPP, QImage::Format_RGBA8888, (QImageCleanupFunction) &cleanUpBuffer, (void*) qbuffer));
 
 	// Update height and width
 	width = image->width();
@@ -956,7 +986,9 @@ void Frame::cleanUpBuffer(void *info)
 // Add audio silence
 void Frame::AddAudioSilence(int numSamples)
 {
-	// Resize audio container
+    const GenericScopedLock<CriticalSection> lock(addingAudioSection);
+
+    // Resize audio container
 	audio->setSize(channels, numSamples, false, true, false);
 	audio->clear();
 	has_audio_data = true;
