@@ -31,7 +31,7 @@ using namespace std;
 using namespace openshot;
 
 FrameMapper::FrameMapper(ReaderBase *reader, Fraction target, PulldownType target_pulldown, int target_sample_rate, int target_channels, ChannelLayout target_channel_layout) :
-		reader(reader), target(target), pulldown(target_pulldown), is_dirty(true), avr(NULL)
+		reader(reader), target(target), pulldown(target_pulldown), is_dirty(true), avr(NULL), timeline_frame_offset(0)
 {
 	// Set the original frame rate from the reader
 	original = Fraction(reader->info.fps.num, reader->info.fps.den);
@@ -66,7 +66,17 @@ FrameMapper::~FrameMapper() {
 		Close();
 }
 
-void FrameMapper::AddField(long int frame)
+/// Get the current reader
+ReaderBase* FrameMapper::Reader()
+{
+    if (reader)
+        return reader;
+    else
+        // Throw error if reader not initialized
+        throw ReaderClosed("No Reader has been initialized for FrameMapper.  Call Reader(*reader) before calling this method.", "");
+}
+
+void FrameMapper::AddField(int64_t frame)
 {
 	// Add a field, and toggle the odd / even field
 	AddField(Field(frame, field_toggle));
@@ -126,11 +136,11 @@ void FrameMapper::Init()
 
 
 		// Calculate # of fields to map
-		long int frame = 1;
-		long int number_of_fields = reader->info.video_length * 2;
+		int64_t frame = 1;
+		int64_t number_of_fields = reader->info.video_length * 2;
 
 		// Loop through all fields in the original video file
-		for (long int field = 1; field <= number_of_fields; field++)
+		for (int64_t field = 1; field <= number_of_fields; field++)
 		{
 
 			if (difference == 0) // Same frame rate, NO pull-down or special techniques required
@@ -197,8 +207,8 @@ void FrameMapper::Init()
 	} else {
 		// Map the remaining framerates using a simple Keyframe curve
 		// Calculate the difference (to be used as a multiplier)
-		float rate_diff = target.ToFloat() / original.ToFloat();
-		long int new_length = reader->info.video_length * rate_diff;
+		double rate_diff = target.ToDouble() / original.ToDouble();
+		int64_t new_length = reader->info.video_length * rate_diff;
 
 		// Build curve for framerate mapping
 		Keyframe rate_curve;
@@ -206,7 +216,7 @@ void FrameMapper::Init()
 		rate_curve.AddPoint(new_length, reader->info.video_length, LINEAR);
 
 		// Loop through curve, and build list of frames
-		for (long int frame_num = 1; frame_num <= new_length; frame_num++)
+		for (int64_t frame_num = 1; frame_num <= new_length; frame_num++)
 		{
 			// Add 2 fields per frame
 			AddField(rate_curve.GetInt(frame_num));
@@ -219,10 +229,10 @@ void FrameMapper::Init()
 	Field Even(0, true);	// temp field used to track the EVEN field
 
 	// Variables used to remap audio samples
-	int start_samples_frame = 1;
+	int64_t start_samples_frame = 1;
 	int start_samples_position = 0;
 
-	for (long int field = 1; field <= fields.size(); field++)
+	for (int64_t field = 1; field <= fields.size(); field++)
 	{
 		// Get the current field
 		Field f = fields[field - 1];
@@ -231,7 +241,7 @@ void FrameMapper::Init()
 		if (field % 2 == 0 && field > 0)
 		{
 			// New frame number
-			long int frame_number = field / 2;
+			int64_t frame_number = field / 2 + timeline_frame_offset;
 
 			// Set the bottom frame
 			if (f.isOdd)
@@ -242,7 +252,7 @@ void FrameMapper::Init()
 			// Determine the range of samples (from the original rate). Resampling happens in real-time when
 			// calling the GetFrame() method. So this method only needs to redistribute the original samples with
 			// the original sample rate.
-			int end_samples_frame = start_samples_frame;
+			int64_t end_samples_frame = start_samples_frame;
 			int end_samples_position = start_samples_position;
 			int remaining_samples = Frame::GetSamplesPerFrame(frame_number, target, reader->info.sample_rate, reader->info.channels);
 
@@ -255,7 +265,7 @@ void FrameMapper::Init()
 				if (original_samples >= remaining_samples)
 				{
 					// Take all that we need, and break loop
-					end_samples_position += remaining_samples;
+					end_samples_position += remaining_samples - 1;
 					remaining_samples = 0;
 				} else
 				{
@@ -298,7 +308,7 @@ void FrameMapper::Init()
 	fields.clear();
 }
 
-MappedFrame FrameMapper::GetMappedFrame(long int TargetFrameNumber) throw(OutOfBoundsFrame)
+MappedFrame FrameMapper::GetMappedFrame(int64_t TargetFrameNumber)
 {
 	// Ignore mapping on single image readers
 	if (info.has_video and !info.has_audio and info.has_single_image) {
@@ -331,12 +341,12 @@ MappedFrame FrameMapper::GetMappedFrame(long int TargetFrameNumber) throw(OutOfB
 }
 
 // Get or generate a blank frame
-tr1::shared_ptr<Frame> FrameMapper::GetOrCreateFrame(long int number)
+std::shared_ptr<Frame> FrameMapper::GetOrCreateFrame(int64_t number)
 {
-	tr1::shared_ptr<Frame> new_frame;
+	std::shared_ptr<Frame> new_frame;
 
 	// Init some basic properties about this frame (keep sample rate and # channels the same as the original reader for now)
-	int samples_in_frame = Frame::GetSamplesPerFrame(number, target, reader->info.sample_rate, reader->info.channels);
+	int samples_in_frame = Frame::GetSamplesPerFrame(number + timeline_frame_offset, target, reader->info.sample_rate, reader->info.channels);
 
 	try {
 		// Debug output
@@ -363,17 +373,17 @@ tr1::shared_ptr<Frame> FrameMapper::GetOrCreateFrame(long int number)
 	ZmqLogger::Instance()->AppendDebugMethod("FrameMapper::GetOrCreateFrame (create blank)", "number", number, "samples_in_frame", samples_in_frame, "", -1, "", -1, "", -1, "", -1);
 
 	// Create blank frame
-	new_frame = tr1::shared_ptr<Frame>(new Frame(number, info.width, info.height, "#000000", samples_in_frame, reader->info.channels));
+	new_frame = std::make_shared<Frame>(number, info.width, info.height, "#000000", samples_in_frame, reader->info.channels);
 	new_frame->SampleRate(reader->info.sample_rate);
 	new_frame->ChannelsLayout(info.channel_layout);
 	return new_frame;
 }
 
 // Get an openshot::Frame object for a specific frame number of this reader.
-tr1::shared_ptr<Frame> FrameMapper::GetFrame(long int requested_frame) throw(ReaderClosed)
+std::shared_ptr<Frame> FrameMapper::GetFrame(int64_t requested_frame)
 {
 	// Check final cache, and just return the frame (if it's available)
-	tr1::shared_ptr<Frame> final_frame = final_cache.GetFrame(requested_frame);
+	std::shared_ptr<Frame> final_frame = final_cache.GetFrame(requested_frame);
 	if (final_frame) return final_frame;
 
 	// Create a scoped lock, allowing only a single thread to run the following code at one time
@@ -389,139 +399,183 @@ tr1::shared_ptr<Frame> FrameMapper::GetFrame(long int requested_frame) throw(Rea
 	if (final_frame) return final_frame;
 
 	// Minimum number of frames to process (for performance reasons)
-	int minimum_frames = OPEN_MP_NUM_PROCESSORS;
-
-	// Set the number of threads in OpenMP
-	omp_set_num_threads(OPEN_MP_NUM_PROCESSORS);
-	// Allow nested OpenMP sections
-	omp_set_nested(true);
+	// Dialing this down to 1 for now, as it seems to improve performance, and reduce export crashes
+	int minimum_frames = 1;
 
 	// Debug output
 	ZmqLogger::Instance()->AppendDebugMethod("FrameMapper::GetFrame (Loop through frames)", "requested_frame", requested_frame, "minimum_frames", minimum_frames, "", -1, "", -1, "", -1, "", -1);
 
-	#pragma omp parallel
+	// Loop through all requested frames
+	for (int64_t frame_number = requested_frame; frame_number < requested_frame + minimum_frames; frame_number++)
 	{
-		// Loop through all requested frames, each frame gets it's own thread
-		#pragma omp for ordered firstprivate(requested_frame, minimum_frames)
-		for (long int frame_number = requested_frame; frame_number < requested_frame + minimum_frames; frame_number++)
+
+		// Debug output
+		ZmqLogger::Instance()->AppendDebugMethod("FrameMapper::GetFrame (inside omp for loop)", "frame_number", frame_number, "minimum_frames", minimum_frames, "requested_frame", requested_frame, "", -1, "", -1, "", -1);
+
+		// Get the mapped frame
+		MappedFrame mapped = GetMappedFrame(frame_number);
+		std::shared_ptr<Frame> mapped_frame;
+
+		// Get the mapped frame (keeping the sample rate and channels the same as the original... for the moment)
+		mapped_frame = GetOrCreateFrame(mapped.Odd.Frame);
+
+		// Get # of channels in the actual frame
+		int channels_in_frame = mapped_frame->GetAudioChannelsCount();
+		int samples_in_frame = Frame::GetSamplesPerFrame(frame_number + timeline_frame_offset, target, mapped_frame->SampleRate(), channels_in_frame);
+
+		// Determine if mapped frame is identical to source frame
+		// including audio sample distribution according to mapped.Samples,
+		// and frame_number. In some cases such as end of stream, the reader
+		// will return a frame with a different frame number. In these cases,
+		// we cannot use the frame as is, nor can we modify the frame number,
+		// otherwise the reader's cache object internals become invalid.
+		if (info.sample_rate == mapped_frame->SampleRate() &&
+			info.channels == mapped_frame->GetAudioChannelsCount() &&
+			info.channel_layout == mapped_frame->ChannelsLayout() &&
+			mapped.Samples.total == mapped_frame->GetAudioSamplesCount() &&
+			mapped.Samples.frame_start == mapped.Odd.Frame &&
+			mapped.Samples.sample_start == 0 &&
+			mapped_frame->number == frame_number &&// in some conditions (e.g. end of stream)
+			info.fps.num == reader->info.fps.num &&
+			info.fps.den == reader->info.fps.den) {
+				// Add original frame to cache, and skip the rest (for performance reasons)
+				final_cache.Add(mapped_frame);
+				continue;
+		}
+
+		// Create a new frame
+		std::shared_ptr<Frame> frame = std::make_shared<Frame>(frame_number, 1, 1, "#000000", samples_in_frame, channels_in_frame);
+		frame->SampleRate(mapped_frame->SampleRate());
+		frame->ChannelsLayout(mapped_frame->ChannelsLayout());
+
+
+		// Copy the image from the odd field
+		std::shared_ptr<Frame> odd_frame;
+		odd_frame = GetOrCreateFrame(mapped.Odd.Frame);
+
+		if (odd_frame)
+			frame->AddImage(std::shared_ptr<QImage>(new QImage(*odd_frame->GetImage())), true);
+		if (mapped.Odd.Frame != mapped.Even.Frame) {
+			// Add even lines (if different than the previous image)
+			std::shared_ptr<Frame> even_frame;
+			even_frame = GetOrCreateFrame(mapped.Even.Frame);
+			if (even_frame)
+				frame->AddImage(std::shared_ptr<QImage>(new QImage(*even_frame->GetImage())), false);
+		}
+
+		// Resample audio on frame (if needed)
+		bool need_resampling = false;
+		if (info.has_audio &&
+			(info.sample_rate != frame->SampleRate() ||
+			 info.channels != frame->GetAudioChannelsCount() ||
+			 info.channel_layout != frame->ChannelsLayout()))
+			// Resample audio and correct # of channels if needed
+			need_resampling = true;
+
+		// create a copy of mapped.Samples that will be used by copy loop
+		SampleRange copy_samples = mapped.Samples;
+
+		if (need_resampling)
 		{
+			// Resampling needed, modify copy of SampleRange object that
+			// includes some additional input samples on first iteration,
+			// and continues the offset to ensure that the sample rate
+			// converter isn't input limited.
+			const int EXTRA_INPUT_SAMPLES = 20;
 
-			// Debug output
-			ZmqLogger::Instance()->AppendDebugMethod("FrameMapper::GetFrame (inside omp for loop)", "frame_number", frame_number, "minimum_frames", minimum_frames, "requested_frame", requested_frame, "", -1, "", -1, "", -1);
-
-			// Get the mapped frame
-			MappedFrame mapped = GetMappedFrame(frame_number);
-			tr1::shared_ptr<Frame> mapped_frame;
-
-            // Get the mapped frame (keeping the sample rate and channels the same as the original... for the moment)
-			mapped_frame = GetOrCreateFrame(mapped.Odd.Frame);
-
-			// Get # of channels in the actual frame
-			int channels_in_frame = mapped_frame->GetAudioChannelsCount();
-			int samples_in_frame = Frame::GetSamplesPerFrame(frame_number, target, mapped_frame->SampleRate(), channels_in_frame);
-
-			// Determine if mapped frame is identical to source frame
-			if (info.sample_rate == mapped_frame->SampleRate() &&
-				info.channels == mapped_frame->GetAudioChannelsCount() &&
-				info.channel_layout == mapped_frame->ChannelsLayout() &&
-				info.fps.num == reader->info.fps.num &&
-				info.fps.den == reader->info.fps.den) {
-					// Set frame # on mapped frame
-					mapped_frame->SetFrameNumber(frame_number);
-
-					// Add original frame to cache, and skip the rest (for performance reasons)
-					final_cache.Add(mapped_frame);
-					continue;
-			}
-
-			// Create a new frame
-			tr1::shared_ptr<Frame> frame = tr1::shared_ptr<Frame>(new Frame(frame_number, 1, 1, "#000000", samples_in_frame, channels_in_frame));
-			frame->SampleRate(mapped_frame->SampleRate());
-			frame->ChannelsLayout(mapped_frame->ChannelsLayout());
-
-
-			// Copy the image from the odd field
-			tr1::shared_ptr<Frame> odd_frame;
-            #pragma omp ordered
-            odd_frame = GetOrCreateFrame(mapped.Odd.Frame);
-
-            if (odd_frame)
-                frame->AddImage(tr1::shared_ptr<QImage>(new QImage(*odd_frame->GetImage())), true);
-            if (mapped.Odd.Frame != mapped.Even.Frame) {
-                // Add even lines (if different than the previous image)
-                tr1::shared_ptr<Frame> even_frame;
-                #pragma omp ordered
-                even_frame = GetOrCreateFrame(mapped.Even.Frame);
-                if (even_frame)
-                    frame->AddImage(tr1::shared_ptr<QImage>(new QImage(*even_frame->GetImage())), false);
-            }
-
-			// Copy the samples
-			int samples_copied = 0;
-			int starting_frame = mapped.Samples.frame_start;
-			while (info.has_audio && samples_copied < mapped.Samples.total)
+			// Extend end sample count by an addtional EXTRA_INPUT_SAMPLES samples
+			copy_samples.sample_end += EXTRA_INPUT_SAMPLES;
+			int samples_per_end_frame =
+				Frame::GetSamplesPerFrame(copy_samples.frame_end, original,
+										  reader->info.sample_rate, reader->info.channels);
+			if (copy_samples.sample_end >= samples_per_end_frame)
 			{
-				// Init number of samples to copy this iteration
-				int remaining_samples = mapped.Samples.total - samples_copied;
-				int number_to_copy = 0;
+				// check for wrapping
+				copy_samples.frame_end++;
+				copy_samples.sample_end -= samples_per_end_frame;
+			}
+			copy_samples.total += EXTRA_INPUT_SAMPLES;
 
-				// Loop through each channel
-				for (int channel = 0; channel < channels_in_frame; channel++)
+			if (avr) {
+				// Sample rate conversion has been allocated on this clip, so
+				// this is not the first iteration. Extend start position by
+				// EXTRA_INPUT_SAMPLES to keep step with previous frame
+				copy_samples.sample_start += EXTRA_INPUT_SAMPLES;
+				int samples_per_start_frame =
+					Frame::GetSamplesPerFrame(copy_samples.frame_start, original,
+											  reader->info.sample_rate, reader->info.channels);
+				if (copy_samples.sample_start >= samples_per_start_frame)
 				{
-					// number of original samples on this frame
-					tr1::shared_ptr<Frame> original_frame = GetOrCreateFrame(starting_frame);
-					int original_samples = original_frame->GetAudioSamplesCount();
-
-					if (starting_frame == mapped.Samples.frame_start)
-					{
-						// Starting frame (take the ending samples)
-						number_to_copy = original_samples - mapped.Samples.sample_start;
-						if (number_to_copy > remaining_samples)
-							number_to_copy = remaining_samples;
-
-						// Add samples to new frame
-						frame->AddAudio(true, channel, samples_copied, original_frame->GetAudioSamples(channel) + mapped.Samples.sample_start, number_to_copy, 1.0);
-					}
-					else if (starting_frame > mapped.Samples.frame_start && starting_frame < mapped.Samples.frame_end)
-					{
-						// Middle frame (take all samples)
-						number_to_copy = original_samples;
-						if (number_to_copy > remaining_samples)
-							number_to_copy = remaining_samples;
-
-						// Add samples to new frame
-						frame->AddAudio(true, channel, samples_copied, original_frame->GetAudioSamples(channel), number_to_copy, 1.0);
-					}
-					else
-					{
-						// Ending frame (take the beginning samples)
-						number_to_copy = mapped.Samples.sample_end;
-						if (number_to_copy > remaining_samples)
-							number_to_copy = remaining_samples;
-
-						// Add samples to new frame
-						frame->AddAudio(false, channel, samples_copied, original_frame->GetAudioSamples(channel), number_to_copy, 1.0);
-					}
+					// check for wrapping
+					copy_samples.frame_start++;
+					copy_samples.sample_start -= samples_per_start_frame;
 				}
+				copy_samples.total -= EXTRA_INPUT_SAMPLES;
+			}
+		}
 
-				// increment frame
-				samples_copied += number_to_copy;
-				starting_frame++;
+		// Copy the samples
+		int samples_copied = 0;
+		int64_t starting_frame = copy_samples.frame_start;
+		while (info.has_audio && samples_copied < copy_samples.total)
+		{
+			// Init number of samples to copy this iteration
+			int remaining_samples = copy_samples.total - samples_copied;
+			int number_to_copy = 0;
+
+			// number of original samples on this frame
+			std::shared_ptr<Frame> original_frame = GetOrCreateFrame(starting_frame);
+			int original_samples = original_frame->GetAudioSamplesCount();
+
+			// Loop through each channel
+			for (int channel = 0; channel < channels_in_frame; channel++)
+			{
+				if (starting_frame == copy_samples.frame_start)
+				{
+					// Starting frame (take the ending samples)
+					number_to_copy = original_samples - copy_samples.sample_start;
+					if (number_to_copy > remaining_samples)
+						number_to_copy = remaining_samples;
+
+					// Add samples to new frame
+					frame->AddAudio(true, channel, samples_copied, original_frame->GetAudioSamples(channel) + copy_samples.sample_start, number_to_copy, 1.0);
+				}
+				else if (starting_frame > copy_samples.frame_start && starting_frame < copy_samples.frame_end)
+				{
+					// Middle frame (take all samples)
+					number_to_copy = original_samples;
+					if (number_to_copy > remaining_samples)
+						number_to_copy = remaining_samples;
+
+					// Add samples to new frame
+					frame->AddAudio(true, channel, samples_copied, original_frame->GetAudioSamples(channel), number_to_copy, 1.0);
+				}
+				else
+				{
+					// Ending frame (take the beginning samples)
+					number_to_copy = copy_samples.sample_end + 1;
+					if (number_to_copy > remaining_samples)
+						number_to_copy = remaining_samples;
+
+					// Add samples to new frame
+					frame->AddAudio(false, channel, samples_copied, original_frame->GetAudioSamples(channel), number_to_copy, 1.0);
+				}
 			}
 
-			// Resample audio on frame (if needed)
-			if (info.has_audio &&
-				(info.sample_rate != frame->SampleRate() ||
-				 info.channels != frame->GetAudioChannelsCount() ||
-				 info.channel_layout != frame->ChannelsLayout()))
-				// Resample audio and correct # of channels if needed
-				ResampleMappedAudio(frame, mapped.Odd.Frame);
+			// increment frame
+			samples_copied += number_to_copy;
+			starting_frame++;
+		}
 
-			// Add frame to final cache
-			final_cache.Add(frame);
+		// Resample audio on frame (if needed)
+		if (need_resampling)
+			// Resample audio and correct # of channels if needed
+			ResampleMappedAudio(frame, mapped.Odd.Frame);
 
-		} // for loop
-	} // omp parallel
+		// Add frame to final cache
+		final_cache.Add(frame);
+
+	} // for loop
 
 	// Return processed openshot::Frame
 	return final_cache.GetFrame(requested_frame);
@@ -570,7 +624,7 @@ bool FrameMapper::IsOpen() {
 
 
 // Open the internal reader
-void FrameMapper::Open() throw(InvalidFile)
+void FrameMapper::Open()
 {
 	if (reader)
 	{
@@ -623,7 +677,7 @@ Json::Value FrameMapper::JsonValue() {
 }
 
 // Load JSON string into this object
-void FrameMapper::SetJson(string value) throw(InvalidJSON) {
+void FrameMapper::SetJson(string value) {
 
 	// Parse JSON string into JSON objects
 	Json::Value root;
@@ -646,7 +700,7 @@ void FrameMapper::SetJson(string value) throw(InvalidJSON) {
 }
 
 // Load Json::JsonValue into this object
-void FrameMapper::SetJsonValue(Json::Value root) throw(InvalidFile) {
+void FrameMapper::SetJsonValue(Json::Value root) {
 
 	// Set parent data
 	ReaderBase::SetJsonValue(root);
@@ -662,13 +716,18 @@ void FrameMapper::SetJsonValue(Json::Value root) throw(InvalidFile) {
 // Change frame rate or audio mapping details
 void FrameMapper::ChangeMapping(Fraction target_fps, PulldownType target_pulldown,  int target_sample_rate, int target_channels, ChannelLayout target_channel_layout)
 {
-	ZmqLogger::Instance()->AppendDebugMethod("FrameMapper::ChangeMapping", "target_fps.num", target_fps.num, "target_fps.den", target_fps.num, "target_pulldown", target_pulldown, "target_sample_rate", target_sample_rate, "target_channels", target_channels, "target_channel_layout", target_channel_layout);
+	ZmqLogger::Instance()->AppendDebugMethod("FrameMapper::ChangeMapping", "target_fps.num", target_fps.num, "target_fps.den", target_fps.den, "target_pulldown", target_pulldown, "target_sample_rate", target_sample_rate, "target_channels", target_channels, "target_channel_layout", target_channel_layout);
 
 	// Mark as dirty
 	is_dirty = true;
 
 	// Update mapping details
-	target = target_fps;
+	target.num = target_fps.num;
+	target.den = target_fps.den;
+	info.fps.num = target_fps.num;
+	info.fps.den = target_fps.den;
+	info.video_timebase.num = target_fps.den;
+	info.video_timebase.den = target_fps.num;
 	pulldown = target_pulldown;
 	info.sample_rate = target_sample_rate;
 	info.channels = target_channels;
@@ -686,10 +745,25 @@ void FrameMapper::ChangeMapping(Fraction target_fps, PulldownType target_pulldow
 		avresample_free(&avr);
 		avr = NULL;
 	}
+
+	// Re-init mapping
+	Init();
+}
+
+// Set offset relative to parent timeline
+void FrameMapper::SetTimelineFrameOffset(int64_t offset)
+{
+	ZmqLogger::Instance()->AppendDebugMethod("FrameMapper::SetTimelineFrameOffset", "offset", offset, "", -1, "", -1, "", -1, "", -1, "", -1);
+
+	// Mark as dirty
+	is_dirty = true;
+
+	// Used to correctly align audio sample distribution
+	timeline_frame_offset = offset;
 }
 
 // Resample audio and map channels (if needed)
-void FrameMapper::ResampleMappedAudio(tr1::shared_ptr<Frame> frame, long int original_frame_number)
+void FrameMapper::ResampleMappedAudio(std::shared_ptr<Frame> frame, int64_t original_frame_number)
 {
 	// Init audio buffers / variables
 	int total_frame_samples = 0;
@@ -739,7 +813,7 @@ void FrameMapper::ResampleMappedAudio(tr1::shared_ptr<Frame> frame, long int ori
 	}
 
 	// Update total samples & input frame size (due to bigger or smaller data types)
-	total_frame_samples = Frame::GetSamplesPerFrame(frame->number, target, info.sample_rate, info.channels);
+	total_frame_samples = Frame::GetSamplesPerFrame(frame->number + timeline_frame_offset, target, info.sample_rate, info.channels);
 
 	ZmqLogger::Instance()->AppendDebugMethod("FrameMapper::ResampleMappedAudio (adjust # of samples)", "total_frame_samples", total_frame_samples, "info.sample_rate", info.sample_rate, "sample_rate_in_frame", sample_rate_in_frame, "info.channels", info.channels, "channels_in_frame", channels_in_frame, "original_frame_number", original_frame_number);
 
@@ -752,33 +826,29 @@ void FrameMapper::ResampleMappedAudio(tr1::shared_ptr<Frame> frame, long int ori
 	ZmqLogger::Instance()->AppendDebugMethod("FrameMapper::ResampleMappedAudio (preparing for resample)", "in_sample_fmt", AV_SAMPLE_FMT_S16, "out_sample_fmt", AV_SAMPLE_FMT_S16, "in_sample_rate", sample_rate_in_frame, "out_sample_rate", info.sample_rate, "in_channels", channels_in_frame, "out_channels", info.channels);
 
 	int nb_samples = 0;
-	// Force the audio resampling to happen in order (1st thread to last thread), so the waveform
-	// is smooth and continuous.
-	#pragma omp ordered
-	{
-		// setup resample context
-		if (!avr) {
-			avr = avresample_alloc_context();
-			av_opt_set_int(avr,  "in_channel_layout", channel_layout_in_frame, 0);
-			av_opt_set_int(avr, "out_channel_layout", info.channel_layout, 0);
-			av_opt_set_int(avr,  "in_sample_fmt",     AV_SAMPLE_FMT_S16,     0);
-			av_opt_set_int(avr, "out_sample_fmt",     AV_SAMPLE_FMT_S16,     0);
-			av_opt_set_int(avr,  "in_sample_rate",    sample_rate_in_frame,    0);
-			av_opt_set_int(avr, "out_sample_rate",    info.sample_rate,    0);
-			av_opt_set_int(avr,  "in_channels",       channels_in_frame,    0);
-			av_opt_set_int(avr, "out_channels",       info.channels,    0);
-			avresample_open(avr);
-		}
 
-		// Convert audio samples
-		nb_samples = avresample_convert(avr, 	// audio resample context
-				audio_converted->data, 			// output data pointers
-				audio_converted->linesize[0], 	// output plane size, in bytes. (0 if unknown)
-				audio_converted->nb_samples,	// maximum number of samples that the output buffer can hold
-				audio_frame->data,				// input data pointers
-				audio_frame->linesize[0],		// input plane size, in bytes (0 if unknown)
-				audio_frame->nb_samples);		// number of input samples to convert
-	}
+    // setup resample context
+    if (!avr) {
+        avr = avresample_alloc_context();
+        av_opt_set_int(avr,  "in_channel_layout", channel_layout_in_frame, 0);
+        av_opt_set_int(avr, "out_channel_layout", info.channel_layout, 0);
+        av_opt_set_int(avr,  "in_sample_fmt",     AV_SAMPLE_FMT_S16,     0);
+        av_opt_set_int(avr, "out_sample_fmt",     AV_SAMPLE_FMT_S16,     0);
+        av_opt_set_int(avr,  "in_sample_rate",    sample_rate_in_frame,    0);
+        av_opt_set_int(avr, "out_sample_rate",    info.sample_rate,    0);
+        av_opt_set_int(avr,  "in_channels",       channels_in_frame,    0);
+        av_opt_set_int(avr, "out_channels",       info.channels,    0);
+        avresample_open(avr);
+    }
+
+    // Convert audio samples
+    nb_samples = avresample_convert(avr, 	// audio resample context
+            audio_converted->data, 			// output data pointers
+            audio_converted->linesize[0], 	// output plane size, in bytes. (0 if unknown)
+            audio_converted->nb_samples,	// maximum number of samples that the output buffer can hold
+            audio_frame->data,				// input data pointers
+            audio_frame->linesize[0],		// input plane size, in bytes (0 if unknown)
+            audio_frame->nb_samples);		// number of input samples to convert
 
 	// Create a new array (to hold all resampled S16 audio samples)
 	int16_t* resampled_samples = new int16_t[(nb_samples * info.channels)];
